/***************************************************************************
 *   Copyright (C) 2005 by Jeff Ferr                                       *
 *   root@sat                                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 a***************************************************************************/
#ifndef J_CONSOLE_H
#define J_CONSOLE_H

#include "jobject.h"

#include <string>

#include <stdint.h>

namespace jcommon {

/**
 * \brief
 * 
 * \author Jeff Ferr
 */
class System : public jcommon::Object{

	private:
		/** \brief */
		static System *_instance;
		/** \brief */
		bool _keyboard;
		
	public:
		/**
		* \brief
		*
		*/
		System();
		
		/**
		* \brief
		*
		*/
		virtual ~System();

		/**
		 * \brief
		 *
		 */
		static System * GetInstance();
		
		/**
		 * \brief
		 *
		 */
		void Beep(int freq, int delay);
		
		/**
		 * \brief
		 *
		 */
		std::string GetCurrentUserName();
		
		/**
		 * \brief
		 *
		 */
		void Exit(int i);

		/**
		 * \brief
		 *
		 */
		void Abort();

		/**
		 * \brief
		 *
		 */
		int GetUserID();
		
		/**
		 * \brief
		 *
		 */
		int GetProcessID();
		
		/**
		 * \brief
		 *
		 */
		std::string GetHomeDirectory();

		/**
		 * \brief
		 *
		 */
		std::string GetEnviromentVariable(std::string key_, std::string default_);
		
		/**
		 * \brief
		 *
		 */
		void UnsetEnviromentVariable(std::string key_);
			
		/**
		 * \brief
		 *
		 */
		int SetEnviromentVariable(std::string key_, std::string value_, bool overwrite_ = true);

		/**
		 * \brief
		 *
		 */
		int GetLastErrorCode();
		
		/**
		 * \brief Error message according with GetLastErrorCode() function.
		 *
		 */
		std::string GetLastErrorMessage();
		
		/**
		 * \brief
		 *
		 */
		void EnableKeyboardBuffer(bool b);

		/**
		 * \brief
		 *
		 */
		void EnableEcho(bool b);

		/**
		 * \brief
		 *
		 */
		int KbHit(void);
		
		/**
		 * \brief
		 *
		 */
		int Getch(void);

		/**
		 * \brief Expand a pathname getting rid of special shell characters like ~.$, etc.
		 * For Unix/Win32 compatibility use $(XXX) instead of $XXX when using
		 * environment variables in a pathname. If compatibility is not an issue
		 * you can use on Unix directly $XXX. This is a protected function called
		 * from the OS specific system classes, like TUnixSystem and TWinNTSystem.
		 *
		 */
		std::string ExpandFileName(std::string fname);

		/**
		 * \brief Reset the system.
		 *
		 */
		int ResetSystem();
		
		/**
		 * \brief 
		 *
		 */
		void Logout();
		
		/**
		 * \brief Shutdown the system.
		 *
		 */
		int ShutdownSystem();
		
		/**
		 * \brief Reset the current program.
		 *
		 */
		void ResetProgram(std::string program, char **argv, char **envp);
		
};

}

#endif
