/***************************************************************************
 *   Copyright (C) 2005 by Jeff Ferr                                       *
 *   root@sat                                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef J_DATAGRAM_SOCKET_H
#define J_DATAGRAM_SOCKET_H

#include "jinetaddress.h"
#include "jdatagramsocketbuffer.h"
#include "jsocketoption.h"

#include <iostream>
#include <string>

#include <netinet/in.h>
#include <sys/types.h>
#include <stdint.h>

namespace jsocket {

/**
 * \brief Socket UDP.
 *
 * \author Jeff Ferr
 */
class DatagramSocket{

    private:
        /** \brief Use to bind the socket in a free port. */
        static uint16_t _used_port;
        
        /** \brief Descriptor. */
        int _fd;
        /** \brief Local socket. */
        sockaddr_in _lsock;
        /** \brief Server socket UDP. */
        sockaddr_in _server_sock;
        /** \brief Local inetaddress. */
        InetAddress *_local;
        /** \brief Remote inetaddress. */
        InetAddress *_address;
        /** \brief Buffer UDP. */
        DatagramSocketBuffer *_buffer;
        /** \brief Socket options. */
        SocketOption *_sock_opt;
        /** \brief Input stream. */
        std::istream *_is;
        /** \brief Output stream. */
        std::ostream *_os;
        /** \brief Bytes sent. */
        uint64_t _sent_bytes;
        /** \brief Bytes received. */
        uint64_t _receive_bytes;
        
        /**
        * \brief Create a new socket.
        *
        */
        void CreateSocket();
        
        /**
        * \brief Bind socket.
        *
        */
        void BindSocket(InetAddress *addr_, uint16_t local_port_);
        
        /**
        * \brief Connect socket.
        *
        */
        void ConnectSocket(InetAddress *addr_, uint16_t port_);
        
        /**
        * \brief Init the stream.
        *
        */
        void InitStream();

    public:
        /**
        * \brief Construtor UDP client.
        *
        */
        DatagramSocket(std::string addr_, uint16_t port_);
        
        /**
        * \brief Construtor UDP server.
        *
        */
        DatagramSocket(uint16_t port_);
        
        /**
        * \brief Destructor virtual.
        *
        */
        virtual ~DatagramSocket();
        
        /**
        * \brief 
        *
        */
        std::istream & GetInputStream();
        
        /**
        * \brief
        *
        */
        std::ostream & GetOutputStream();
        
        /**
        * \brief Read data from a source.
        *
        */
        int Receive(char *data_, int size_);
        
        /**
        * \brief Write data to a source.
        *
        */
        int Send(char *data_, int size_, bool notify_ = false);
        
        /**
        * \brief Close the socket.
        *
        */
        void Close();
        
        
        /**
        * \brief Get socket descriptor.
        *
        */
        int16_t GetHandler();
        
        /**
        * \brief
        *
        */
        sockaddr_in LocalAddress();
        
        /**
        * \brief
        *
        */
        sockaddr_in RemoteAddress();
        
        /**
        * \brief Get InetAddress.
        *
        */
        InetAddress * GetInetAddress();
        
        /**
        * \brief Get the local port.
        *
        */
        uint16_t GetLocalPort();
        
        /**
        * \brief Get port.
        *
        */
        uint16_t GetPort();
        
        /**
        * \brief Get sent bytes to destination.
        *
        */
        uint64_t GetSentBytes();
        
        /**
        * \brief Get received bytes from a source.
        *
        */
        uint64_t GetReceiveBytes();

        /**
         * \brief Get a object SocketOption.
         *
         */
        SocketOption * GetSocketOption();
        
        /**
        * \brief
        *
        */
        std::string what();

};

};

#endif
