/***************************************************************************
 *   Copyright (C) 2005 by Jeff Ferr                                       *
 *   root@sat                                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef J_SOCKET_H
#define J_SOCKET_H

#include "jinetaddress.h"
#include "jsocketbuffer.h"
#include "jserversocket.h"
#include "jsocketoption.h"

#include <iostream>

#include <unistd.h>
#include <sys/socket.h>
#include <stdint.h>

namespace jsocket {

class ServerSocket;

/**
 * \brief Socket.
 *
 * \author Jeff Ferr
 */
class Socket{

    friend class ServerSocket; //Socket * ServerSocket::Accept();

    private:
        /** \brief Socket handler. */
        int16_t _fd;
        /** \brief Socket buffer. */
        SocketBuffer *_buffer;
        /** \brief Socket options. */
        SocketOption *_sock_opt;
        /** \brief */
        std::istream *_is;
        /** \brief */
        std::ostream *_os;
        /** \brief */
        sockaddr_in _lsock;
        /** \brief */
        sockaddr_in _server_sock;
        /** \brief */
        InetAddress *_local;
        /** \brief */
        InetAddress *_address;
        /** \brief Bytes sent. */
        uint64_t _sent_bytes;
        /** \brief Bytes received. */
        uint64_t _receive_bytes;

        /**
        * \brief
        *
        */
        /* Socket(uint16_t handler_, sockaddr_in server_); */
        
        /**
        * \brief Create a new socket.
        *
        */
        void CreateSocket();
        
        /**
        * \brief
        *
        */
        void BindSocket(InetAddress *, uint16_t);
        
        /**
        * \brief Connect the socket.
        *
        */
        void ConnectSocket(InetAddress *, uint16_t);
        
        /**
        * \brief
        *
        */
        void InitStreams();

    public:
	/**
	 * \brief Constructor.
	 *
	 */
        Socket(uint16_t handler_, sockaddr_in server_);
	
        /**
        * \brief Constructor.
        *
        */
        Socket(InetAddress *addr_, uint16_t port_);
        
        /**
        * \brief Constructor.
        *
        */
        Socket(InetAddress *addr_, uint16_t port_, InetAddress *local_addr_, uint16_t local_port_);
        
        /**
        * \brief
        *
        */
        Socket(std::string host_, uint16_t port_);
        
        /**
        * \brief Constructor.
        *
        */
        Socket(std::string host_, uint16_t port_, InetAddress *local_addr_, uint16_t local_port_);
        
        /**
        * \brief Destrutor virtual.
        *
        */
        virtual ~Socket();
        
        /**
        * \brief Send bytes to a destination.
        *
        */
        uint16_t Send(char *b_, int size_);
        
        /**
        * \brief Receive bytes from a source.
        *
        */
        uint16_t Receive(char *b_, int size_);

        /**
         * \brief Get the socket handler.
         *
         */
        int16_t GetHandler();
        
        /**
        * \brief Close the socket.
        *
        */
        void Close();
        
        /**
        * \brief
        *
        */
        std::istream & GetInputStream();
        
        /**
        * \brief
        *
        */
        std::ostream & GetOutputStream();
        
        /**
        * \brief
        *
        */
        InetAddress * GetInetAddress();
        
        /**
        * \brief Get the local adrress.
        *
        */
        InetAddress * GetLocalAddress();
        
        /**
        * \brief Get the local port.
        *
        */
        uint16_t GetLocalPort();
        
        /**
        * \brief Get the port.
        *
        */
        uint16_t GetPort();

        /**
        * \brief Get the bytes sent to a destination.
        *
        */
        uint64_t GetSentBytes();
            
        /**
        * \brief Get de bytes received from a source.
        *
        */
        uint64_t GetReceiveBytes();

        /**
         * \brief Get the socket options.
         *
         */
        SocketOption * GetSocketOption();
        
        /**
        * \brief
        *
        */
        std::string what();

};

};

#endif
