/***************************************************************************
 *   Copyright (C) 2005 by Jeff Ferr                                       *
 *   root@sat                                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef J_THREAD_H
#define J_THREAD_H

#include "jcondition.h"
#include "jthreadexception.h"

namespace jthread{

enum thread_policy_t {
	POLICY_OTHER = SCHED_OTHER,		// Normal thread
	POLICY_FIFO = SCHED_FIFO,		// Real-time thread fifo
	POLICY_ROUND_ROBIN = SCHED_RR	// Round robin thread round robin
};

enum thread_priority_t {
	LOW_PRIORITY = 0,
	NORMAL_PRIORITY = 5,
	HIGH_PRIORITY = 10
};

/**
 * \brief Thread.
 *
 * @author Jeff Ferr
 */
class Thread{

    private:
        /** \brief */
        pthread_t _thread;
        /** \brief */
        Condition _condition;
        /** \brief */
        bool _is_running;
        
        /**
         * \brief
         *
         */
        static void * ThreadMain(void *);
        
        /**
         * \brief Notify the end of thread.
         *
         */
        void SignalThreadDead();

    protected:
	/**
	 * \brief
	 *
	 */
	virtual int SetUp();
    
        /**
         * \brief
         *
         */
        virtual void Run();
        
        /**
         * \brief
         *
         */
        virtual int CleanUp();

    public:
        /**
         * \brief Construtor.
         *
         */
        Thread();

        /**
         * \brief Destrutor virtual.
         *
         */
        virtual ~Thread();

        /**
         * \brief Init the thread.
         *
         */
        void Start() throw(ThreadException);
       
        /**
         * \brief Interrupt the thread.
         *
         */
        void Interrupt() throw(ThreadException);
    
	/**
	 * \brief Suspend the thread.
	 *
	 */
	void Suspend();

	/**
	 * \brief Resume the thread.
	 *
	 */
	void Resume();
 
        /**
         * \brief Return true if thread is started, false if not.
         *
         */
        bool IsStarted();
        
        /**
         * \bried
         *
         */
        void SetPolicy(thread_policy_t policy, thread_priority_t priority) throw(ThreadException);
        
        /**
         * \bried
         *
         */
        void GetPolicy(thread_policy_t *policy, thread_priority_t *priority) throw(ThreadException);
        
        /**
         * \brief Wait for end of thread.
         *
         */
        void WaitThread();
       
};

};

#endif
